"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileTester = exports.Type = exports.Status = void 0;
const chalk_1 = __importDefault(require("chalk"));
const typescript_1 = __importDefault(require("typescript"));
const Logger_1 = require("./Logger");
var Status;
(function (Status) {
    Status["pass"] = "pass";
    Status["fail"] = "fail";
})(Status || (exports.Status = Status = {}));
var Type;
(function (Type) {
    Type["group"] = "group";
    Type["test"] = "test";
})(Type || (exports.Type = Type = {}));
class FileTester {
    program;
    sourceFile;
    testsResults = [];
    constructor(program, sourceFile) {
        this.program = program;
        this.sourceFile = sourceFile;
    }
    getRecieveAndExpected(node) {
        const typeChecker = this.program.getTypeChecker();
        let recieveTypeString = null;
        if (typescript_1.default.isPropertyAccessExpression(node.expression)) {
            const leftExpression = node.expression.expression;
            const assertType = typeChecker.getTypeAtLocation(leftExpression);
            if (assertType?.resolvedTypeArguments?.[0]) {
                recieveTypeString = typeChecker.typeToString(assertType.resolvedTypeArguments[0]);
            }
        }
        const ExpectedType = node.typeArguments?.[0] ? typeChecker.getTypeFromTypeNode(node.typeArguments[0]) : null;
        return {
            recieve: recieveTypeString || 'Error displaying recieve type, please check it in the code manually.',
            expected: ExpectedType ? typeChecker.typeToString(ExpectedType) : null,
        };
    }
    visitAssert(result, node) {
        if (typescript_1.default.isCallExpression(node)) {
            const typeChecker = this.program.getTypeChecker();
            const signature = typeChecker.getResolvedSignature(node);
            const returnType = signature ? typeChecker.typeToString(typeChecker.getReturnTypeOfSignature(signature)) : null;
            if (!returnType || !returnType.includes('RTT_FAIL')) {
                return;
            }
            const errorMessage = returnType.slice(returnType.indexOf('<') + 2, -2);
            const simpleExpected = errorMessage.slice(errorMessage.indexOf('`') + 1, -1);
            const { recieve, expected } = this.getRecieveAndExpected(node);
            Object.assign(result, {
                status: Status.fail,
                start: node.getStart(),
                length: node.getWidth(),
                reason: [
                    chalk_1.default.red('Recieved:'),
                    (0, Logger_1.indent)(1) + recieve,
                    chalk_1.default.green('Expected:'),
                    (0, Logger_1.indent)(1) + (expected || simpleExpected),
                    '',
                    chalk_1.default.red('Error: ', errorMessage),
                ],
            });
        }
        typescript_1.default.forEachChild(node, this.visitAssert.bind(this, result));
    }
    visitTest(group, node) {
        if (!typescript_1.default.isCallExpression(node) || !typescript_1.default.isIdentifier(node.expression) || !['testType', 'test'].includes(node.expression.text)) {
            return;
        }
        const testResult = {
            type: Type.test,
            status: Status.pass,
            //@ts-ignore
            description: node.arguments[0]?.text || '',
        };
        const cb = node.arguments[1];
        if (!cb) {
            return;
        }
        if (typescript_1.default.isFunctionLike(cb)) {
            this.visitAssert(testResult, cb.body);
        }
        if (typescript_1.default.isArrayLiteralExpression(cb)) {
            cb.elements.forEach((node) => {
                this.visitAssert(testResult, node);
            });
        }
        if (typescript_1.default.isObjectLiteralExpression(cb)) {
            cb.properties.forEach((node) => {
                if (!typescript_1.default.isPropertyAssignment(node) || !typescript_1.default.isCallExpression(node.initializer)) {
                    return;
                }
                this.visitAssert(testResult, node.initializer);
            });
        }
        if (testResult.status === Status.fail) {
            group.status = Status.fail;
        }
        group.tests.push(testResult);
    }
    visitDescribes(parentGroup, node) {
        if (!typescript_1.default.isCallExpression(node) || !typescript_1.default.isIdentifier(node.expression) || !['describeType', 'describe'].includes(node.expression.text)) {
            return;
        }
        const cb = node.arguments[1];
        if (!cb || !typescript_1.default.isFunctionLike(cb)) {
            return;
        }
        const group = {
            type: Type.group,
            //@ts-ignore
            description: node.arguments[0]?.text || '',
            status: Status.pass,
            tests: [],
        };
        typescript_1.default.forEachChild(cb.body, (node) => {
            typescript_1.default.forEachChild(node, this.visitTest.bind(this, group));
            typescript_1.default.forEachChild(node, this.visitDescribes.bind(this, group));
        });
        if (parentGroup) {
            parentGroup.tests.push(group);
            if (group.status === Status.fail) {
                parentGroup.status = Status.fail;
            }
        }
        else {
            this.testsResults.push(group);
        }
    }
    runTests() {
        typescript_1.default.forEachChild(this.sourceFile, (node) => {
            typescript_1.default.forEachChild(node, this.visitDescribes.bind(this, null));
        });
        return this.testsResults;
    }
}
exports.FileTester = FileTester;
