"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReadableTypesTester = void 0;
const chalk_1 = __importDefault(require("chalk"));
const index_1 = require("../Config/index");
const Logger_1 = require("./Logger");
const SingleFileTester_1 = require("./SingleFileTester");
const typescript_1 = __importDefault(require("typescript"));
class ReadableTypesTester {
    program;
    config = index_1.RTFT_CONFIG;
    totalFiles = 0;
    totalFailFiles = 0;
    totalPassFiles = 0;
    total = 0;
    totalFail = 0;
    totalPass = 0;
    constructor(program) {
        this.program = program;
    }
    printDetails() {
        Logger_1.Logger.log()
            .log(chalk_1.default.whiteBright.bold('Resume:'))
            .log()
            .printRecords()
            .log('Tests:', chalk_1.default.red.bold(this.totalFail, 'Failed'), chalk_1.default.green.bold(this.totalPass, 'Passed'), chalk_1.default.grey.bold(this.total, 'total'))
            .log('Tests Suites:', chalk_1.default.red.bold(this.totalFailFiles, 'Failed'), chalk_1.default.green.bold(this.totalPassFiles, 'Passed'), chalk_1.default.grey.bold(this.totalFiles, 'total'))
            .timeEnd('Time')
            .log();
        if (this.config.verbose) {
            Logger_1.Logger.log('Used configuration:', this.config, '\n');
            Logger_1.Logger.log('Used TS version:', typescript_1.default.version, '\n');
        }
    }
    printFileTestsResults(results, sourceFile, deep = 0) {
        results.forEach((describeResult) => {
            if (describeResult.status === SingleFileTester_1.Status.fail) {
                Logger_1.Logger.record();
                Logger_1.Logger.log((0, Logger_1.indent)(deep + 1) + chalk_1.default.whiteBright(describeResult.description)).endRecord();
            }
            else if (this.config.verbose) {
                Logger_1.Logger.log((0, Logger_1.indent)(deep + 1) + chalk_1.default.whiteBright(describeResult.description)).endRecord();
            }
            describeResult.tests.forEach((testResult) => {
                if (testResult.type === SingleFileTester_1.Type.group) {
                    this.printFileTestsResults([testResult], sourceFile, deep + 1);
                    return;
                }
                this.total++;
                if (testResult.status === SingleFileTester_1.Status.pass) {
                    if (this.config.verbose) {
                        Logger_1.Logger.log((0, Logger_1.indent)(deep + 2) + chalk_1.default.green('✓'), chalk_1.default.grey(testResult.description));
                    }
                    this.totalPass++;
                    return;
                }
                this.totalFail++;
                const errorPosition = sourceFile.getLineAndCharacterOfPosition(testResult.start);
                Logger_1.Logger.record()
                    .log((0, Logger_1.indent)(deep + 2) + chalk_1.default.red('⨯', testResult.description))
                    .log();
                testResult.reason.forEach((line) => {
                    Logger_1.Logger.log((0, Logger_1.indent)(deep + 3) + line);
                });
                Logger_1.Logger.log();
                const startLineToShow = errorPosition.line < 3 ? errorPosition.line : errorPosition.line - 3;
                Array.from({ length: 7 }, (_v, i) => i + startLineToShow).forEach((line) => {
                    const sign = line === errorPosition.line ? chalk_1.default.red.bold('> ') : '  ';
                    Logger_1.Logger.log(chalk_1.default.gray((0, Logger_1.indent)(deep + 2) + `${sign}${line + 1}|` + sourceFile.text.split('\n')[line]));
                });
                Logger_1.Logger
                    .log()
                    .log((0, Logger_1.indent)(deep + 3) + 'at ' + chalk_1.default.blue(`${sourceFile.fileName}:${errorPosition.line + 1}:${errorPosition.character + 1}`))
                    .log()
                    .endRecord();
            });
        });
    }
    runTests() {
        console.time('Time');
        const sourceFiles = this.program.getSourceFiles();
        sourceFiles.forEach((sourceFile) => {
            if (!this.config.include.some((rgx) => rgx.test(sourceFile.fileName))
                || this.config.exclude.some((rgx) => rgx.test(sourceFile.fileName))) {
                return;
            }
            this.totalFiles++;
            const results = this.runIndividualTest(sourceFile);
            if (results.some((result) => result.status === SingleFileTester_1.Status.fail)) {
                this.totalFailFiles++;
                Logger_1.Logger.record()
                    .log(chalk_1.default.bgRed.white(' FAIL '), chalk_1.default.gray(sourceFile.fileName))
                    .endRecord();
            }
            else {
                this.totalPassFiles++;
                Logger_1.Logger.log(chalk_1.default.bgGreen.white(' PASS '), chalk_1.default.gray(sourceFile.fileName));
            }
            this.printFileTestsResults(results, sourceFile);
        });
        this.printDetails();
        return this.totalFail > 0 ? 1 : 0;
    }
    runIndividualTest(sourceFile) {
        return new SingleFileTester_1.FileTester(this.program, sourceFile).runTests();
    }
}
exports.ReadableTypesTester = ReadableTypesTester;
